class WeiboNotExist(Exception):
    pass


class PostWeiboError(Exception):
    pass


class CommentError(Exception):
    pass


class DeleteCommentError(Exception):
    pass


class UploadPicError(Exception):
    pass


class DeleteWeiboError(Exception):
    pass


class SendMessageError(Exception):
    pass


class LikeWeiboError(Exception):
    pass
